package gestionClinica;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.time.LocalDate;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * Esta clase modela una <b>Agenda</b> para todo lo referente a la gestion de <b>Citas</b>
 * entre <b>Pacientes</b> y <b>Psicologos</b> en distintos horarios.<br>
 * La <b>Agenda</b> tendrá una lista de objetos disponibles que representan los dias
 * desde la fecha de la creación de la <b>Agenda</b>, hasta final de este año.<br>
 * Por cada día, habrá 9 objetos, representando las horas entre las 08:00 y las
 * 17:00.<br>Por cada hora será posible concertar tres citas simultáneas ya que 
 * disponemos de tres <b>Psicologos</b>.<br><br>Los <b>Psicologos</b> están declarados en un array
 * como atributo constante de esta clase ya que el enunciado del ejercicio no 
 * exige realizar cambios en ellos y así se simplifica el código.<br>Si se quisieran realizar cambios
 *  como agregar o eliminar <b>Psicologos</b>, debería añadirse una nueva clase específica que se encargase
 * de las gestiones, similar a la clase llamada <b>GestionPacientes</b>
 * 
 * @author Kabuto
 * @version 08/2019
 */
public class Agenda {
	
	public final Psicologo[] PSICOLOGOS; //Los 3 psicologos estarán "hardcoded" y serán constantes
	private ArrayList<HorarioCita> horariosCitas;
	private Scanner teclado;
	
	public Agenda() {
		PSICOLOGOS = new Psicologo[] {new Psicologo("PS001", "Ramiro Sanchez"),
				new Psicologo("PS002", "Carmen Vela"), new Psicologo("PS003", "Pedro Verdún")};
		horariosCitas = new ArrayList<HorarioCita>();
		iniciarAgenda();
		teclado = new Scanner(System.in);
	}
	
	/**
	 * Guarda los datos de la Agenda en un archivo llamado
	 * "agenda.txt"
	 */
	public void guardarAgenda() {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter("agenda.txt", false));
			for (HorarioCita hc: horariosCitas)
				bw.write(hc.toLineaFichero());
			bw.close();
		} catch (IOException e) {
			System.out.println("ERROR al guardar datos de Citas" +
					"\nNo se pudo acceder a fichero agenda.txt");
		}
	}
	
	/**
	 * Intenta recuperar los datos de la Agenda almacenados
	 * en el archivo "agenda.txt"
	 * @return <b>true</b> si logró recuperar datos, <b>false</b> si
	 * no pudo recuperar datos.
	 */
	private boolean recuperarAgenda() {
		try {
			BufferedReader br = new BufferedReader(new FileReader("agenda.txt"));
			String linea = br.readLine();
			while (linea != null) {
				//Primera linea: fecha, hora y número de citas concertadas
				String[] datos = linea.split("<>");
				HorarioCita hc = new HorarioCita(datos[0], datos[1]);
				int numCitas = Integer.parseInt(datos[2]);
				//Siguientes lineas, si las hay, paciente y psicologo citados
				for (int i = 0; i < numCitas; i++) {
					linea = br.readLine();
					String[] citados = linea.split("<>");
					hc.addCita(new Cita(citados[0], citados[1]));
				}
				//Ya tenemos todos los datos, añadimos HorarioCita a la agenda
				horariosCitas.add(hc);
				//Pasamos al siguiente, si es que aún quedan
				linea = br.readLine();
			}
			br.close();
			//Aparentemente, hemos recuperado la agenda desde fichero
			return true;
		} catch (FileNotFoundException e) {
			System.out.println("No existe archivo agenda.txt.\n"
					+ "No hay datos de Citas para recuperar");
			return false;
		} catch (IOException e) {
			System.out.println("Error al leer archivo agenda.txt"
					+ "El archivo está dañado o tiene el acceso restringido.");
			return false;
		}
	}
	
	/**
	 * Este método es llamado desde el constructor de esta clase para
	 * que se encargue de iniciar la Agenda.<br>Primero intentará
	 * recuperar datos de la Agenda desde el fichero "agenda.txt".<br>
	 * Si no lo consigue, entonces creará una nueva lista de HorariosCita
	 * a partir de la fecha actual, hasta final de año.
	 */
	private void iniciarAgenda() {
		System.out.println("\n\nBuscando archivo agenda.txt para recuperar datos previos....\n");
		if (!recuperarAgenda()) {
			/*
			 * Si la recuperacion de la agenda desde fichero txt
			 * ha fracasado, ya sea porque no existe o porque hay
			 * problemas de acceso, lo que haremos es crear una nueva
			 * lista de HorariosCita a partir de la fecha de hoy,
			 * hasta fin de año
			 */
			System.out.println("Se creará una nueva Agenda vacía a partir de la fecha de hoy..");
			LocalDate ahora = LocalDate.now();
			int diaDelAnio = ahora.getDayOfYear();
			final int ULTIMO_DIA;
			if (ahora.isLeapYear()) //Es bisiesto
				ULTIMO_DIA = 366;
			else
				ULTIMO_DIA = 365;
			/*
			 * Por cada dia del año entre hoy hasta fin de año,
			 * haremos horariosCita desde 8AM hasta 4PM(16h)
			 */
			for (int i = diaDelAnio; i <= ULTIMO_DIA; i++) {
				//Cosntruimos nueva fecha según el año actual y el dia indicado por el bucle
				LocalDate date = LocalDate.ofYearDay(ahora.getYear(), i);
				//Con la nueva fecha, construimos un String tipo dd/mm/yyyy
				String fecha = String.format("%02d/%02d/%02d", date.getDayOfMonth(),
						date.getMonthValue(), date.getYear());
				//Recorremos las horas entre la 08 y las 17 y creamos horariosCita junto con la fecha.
				//La última hora de cita serán la 16:00, pues se cierra a las 17:00
				for (int hora = 8; hora < 17; hora++)
					horariosCitas.add(new HorarioCita(fecha, String.format("%02d:00", hora)));
			}
			System.out.println("Nueva Agenda creada");
			//Creamos un nuevo fichero agenda.txt
			guardarAgenda();
		}
		else
			System.out.println("Agenda recuperada con éxito\n");
	}

}
