package gestionClinica;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Scanner;

/**
 * Esta clase se encarga de toda la gestión relativa a los <b>Pacientes</b>.<br>
 * Gestiones como hacer nuevos registros, guardar en disco, recuperar de disco,
 * etc...
 * 
 * @author Kabuto
 * @version 08/2019
 */

public class GestorPacientes {

	private ArrayList<Paciente> pacientes;
	private Scanner teclado;

	public GestorPacientes() {
		pacientes = new ArrayList<Paciente>();
		recuperarPacientes();  //Intentamos recuperar pacientes desde archivo texto
		teclado = new Scanner(System.in);
	}

	/**
	 * Permite al usuario registrar un nuevo <b>Paciente</b>.<br>
	 * Durante el registro se le ha de asignar uno de los <b>Psicólogos</b>
	 * existentes, por eso recibe por parámetros el array con los tres que 
	 * hay disponibles, para mostrar fácilmente sus datos y que el usuario pueda escoger.
	 * <br><br>El nuevo registro será automáticamente guardado en disco.
	 * @param psicologos Array con los tres <b>Psicologos</b> disponibles en la clínica.
	 * @return <b>true</b> si el registro del <b>Paciente</b> tuvo exito.<br>
	 * <b>false</b> si no se pudo hacer el nuevo registro, por ejemplo porque ya
	 * exista un <b>Paciente</b> con la misma <i>cédula</i>.
	 */
	public boolean nuevoPaciente(Psicologo[] psicologos) {
		System.out.println("\n\t\tREGISTRAR NUEVO PACIENTE");
		System.out.println("\t\t--------- ----- --------");
		String cedula = "";
		while (cedula == "") {
			System.out.print("Cédula: ");
			cedula = teclado.nextLine();
		}
		String nombre = "";
		while (nombre == "") {
			System.out.print("Nombre Completo: ");
			nombre = teclado.nextLine();
		}
		String telefono = "";
		while (telefono == "") {
			System.out.print("Teléfono: ");
			telefono = teclado.nextLine();
		}
		String eMail = "";
		while (eMail == "") {
			System.out.print("E-Mail: ");
			eMail = teclado.nextLine();
		}
		Paciente nuevoPaciente = new Paciente(cedula, nombre, telefono, eMail);
		System.out.println("\nA continuación elija un psicólogo para asignar...");
		for (int i = 0; i < 3; i++)
			System.out.println("#" + (i+1) + " " + psicologos[i]);
		System.out.println();
		int opcion = 0;
		while (opcion < 1 || opcion > 3) {
			try {
				System.out.print("Escoja opción [1-3]: ");
				opcion = Integer.parseInt(teclado.nextLine());
			}catch(NumberFormatException ex) {
				System.out.println("Teclee solo números en el rango [1-3]");
			}
		}
		//Ya tenemos escogido un psicologo válido
		nuevoPaciente.setPsicologoAsignado(psicologos[opcion-1].getNumero());
		if (pacientes.contains(nuevoPaciente)) {
			System.out.println("\nYa existe un Paciente con ese número de cédula");
			return false;
		}
		else {
			if (pacientes.add(nuevoPaciente)) {
				guardarPaciente(nuevoPaciente);
				return true;
			}
			else
				return false;
		}
	}

	public void listarPacientes() {
		System.out.println("\n\t\tLISTADO DE PACIENTES");
		System.out.println("\t\t------- -- ---------\n");
		if (pacientes.isEmpty())
			System.out.println("No hay Pacientes registrados para mostrar.");
		else
			for (Paciente paci: pacientes) {
				System.out.println(paci);
				System.out.println("\t\t_____________________________________________");
			}
		pausa();
	}

	public void mostrarExpediente() {
		System.out.println("\n\t\tMOSTRAR EXPEDIENTES");
		System.out.println("\t\t------- -----------\n");
		String cedula = "";
		while (cedula == "") {
			System.out.print("Indique número cédula: ");
			cedula = teclado.nextLine();
		}
		//Buscamos paciente...
		boolean encontrado = false;
		for (Paciente paci: pacientes) {
			if (paci.getCedula().equals(cedula)) {
				encontrado = true;
				paci.listarExpediente();
				pausa();
			}
		}
		if (!encontrado)
			System.out.println("--- No hay Pacientes con la cédula: " + cedula + " ---");
		else
			System.out.println("--- Fin del expediente ---");
		pausa();
	}

	/**
	 * Guarda los datos de todos los <b>Pacientes</b> en disco, en un
	 * archivo llamado <i>pacientes.txt</i>
	 */
	public void guardarPacientes() {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter("pacientes.txt", false));
			for (Paciente paci: pacientes)
				bw.write(paci.toLineaFichero());
			bw.close();
		} catch (IOException e) {
			System.out.println("ERROR al guardar datos de Pacientes" +
					"\nNo se pudo acceder a fichero pacientes.txt");
		}
	}
	
	/**
	 * Guarda los datos del último <b>Paciente</b> registrado en disco.<br>
	 * Lo añade al final del archivo <i>pacientes.txt</i>.
	 * @param paci <b>Paciente</b> que hay que guardar.
	 */
	public void guardarPaciente(Paciente paci) {
		try {
			BufferedWriter bw = new BufferedWriter(new FileWriter("pacientes.txt", true));
			bw.write(paci.toLineaFichero());
			bw.close();
		} catch (IOException e) {
			System.out.println("ERROR al guardar datos de Pacientes" +
					"\nNo se pudo acceder a fichero pacientes.txt");
		}
	}
	
	/**
	 * Intentará recuperar los datos de los <b>Pacientes</b> guardados en disco.
	 */
	private void recuperarPacientes() {
		System.out.println("\n\nBuscando archivo pacientes.txt para recuperar datos previos...\n");
		try {
			BufferedReader br = new BufferedReader(new FileReader("pacientes.txt"));
			String linea = br.readLine();
			while (linea != null) {
				//Primera linea, datos paciente
				String[] datos = linea.split("<>");
				Paciente paci = new Paciente(datos[1], datos[2], datos[3], datos[4]);
				if (!datos[5].equals("null"))
					paci.setHorarioCita(datos[5]);
				if (!datos[6].equals("null"))
					paci.setPsicologoAsignado(datos[6]);
				//Segunda linea, longitud expediente
				linea = br.readLine();
				String[] longitud = linea.split("<>");
				int lng = Integer.parseInt(longitud[1]);
				//Siguientes lineas, si las hay, datos expedientes
				for (int i = 0; i < lng; i++) {
					linea = br.readLine();
					String[] exp = linea.split("<>");
					paci.addExpediente(new Expediente(exp[0], exp[1], exp[2], exp[3]));
				}
				//Paciente completamente recuperado desde fichero
				pacientes.add(paci);
				linea = br.readLine();
			}
			br.close();
			System.out.printf("Se han recuperado a %d pacientes.\n", pacientes.size());
		} catch (FileNotFoundException e) {
			System.out.println("No existe archivo pacientes.txt"
					+ "\nNo hay datos de Pacientes para recuperar.");
		} catch (IOException e) {
			System.out.println("Error al leer archivo pacientes.txt"
					+ "El archivo está dañado o tiene el acceso restringido.");
		}
	}

	/**
	 * Espera a una introduccion por teclado que luego será descartada.<br>
	 * Su propósito es simplemente detener la ejecución del programa hasta que el usuario
	 * intervenga.
	 */
	private void pausa() {
		System.out.println("\n\t\t--- Pulse INTRO para continuar ---");
		teclado.nextLine();
	}

}
